--[[
Name: Crypto-1.0
Revision: $Rev: 18675 $
Author: Sal Scotto (sal.scotto@gmail.com)
Website: http://www.wowace.com
Documentation: need to add wiki entry for it
SVN: $HeadURL: http://svn.wowace.com/wowace/trunk/CryptoLib/Crypto-1.0/Crypto-1.0.lua $
Description: Provides facade interface to lua encryption engines
Dependencies: AceLibrary
]]

local vmajor,vminor = "Crypto-1.0", "$Revision: 18675 $"

if not AceLibrary then error(vmajor .. " requires AceLibrary.") end
if not AceLibrary:IsNewVersion(vmajor, vminor) then return end

local teaLib, rc4Lib, md5


local teaLib = AceLibrary("TEA-1.0")
local rc4lib = AceLibrary("RC4-1.0")
-- this library work with Lua 5.0 and up, some better code can be done with Lua 5.1 but not really an issue right now
local cryptoLib = {}

-- Activate a new instance of the library
local function activate(self, oldLib, oldDeactivate)
	if oldDeactive then oldDeactivate(oldLib) end
end

-- Setup encryption type
function cryptoLib:setCryptoEngine(mode)
	self.mode = "NONE"
	if mode == "RC4" then
		if not AceLibrary:HasInstance("RC4-1.0") then
			return false
		else
			rc4Lib = AceLibrary("RC4-1.0")
		end
		self.mode = "RC4"
		return true
	elseif mode == "TEA" then
		if not AceLibrary:HasInstance("TEA-1.0") and not AceLibrary:HasInstance("MD5-1.0") then
			return false
		else
		 	md5 = AceLibrary("MD5-1.0")
			teaLib = AceLibrary("TEA-1.0")
		end
		self.mode = "TEA"
		return true
	end
	return false
end
-- Encrypt a value
function cryptoLib:encrypt(key,text)	
	if self.mode == "RC4" then
		return rc4Lib:encrypt(key,text)
	elseif mode == "TEA" then
		local skey = teaLib:GenerateKey(key)
		return teaLib:Encrypt(text,key)
	else
		error("Encryption Engine NOT found!")
	end
end

-- Decrypt a value
function cryptoLib:decrypt(key,cipher)
	if self.mode == "RC4" then
		return rc4Lib:decrypt(key,cipher)
	elseif mode == "TEA" then
		local skey = teaLib:GenerateKey(key)
		return teaLib:Decrypt(cipher,key)
	else
		error("Encryption Engine NOT found!")
	end
end

-----------------------
-- Load this library --
-----------------------

AceLibrary:Register(cryptoLib,vmajor,vminor,activate)
cryptoLib = nil